#include "stdafx.h"
#include <shlwapi.h>
#include "HDUSTest.h"
#include "Record.h"
#include "resource.h"




CRecordOption::CRecordOption()
{
	m_szFileName[0]='\0';
	m_ExistsOperation=EXISTS_CONFIRM;
	m_fStopTimeSpec=false;
	m_fStopDateTime=false;
	m_StopTime=60*60*1000;
}


CRecordOption::~CRecordOption()
{
}


bool CRecordOption::SetFileName(LPCTSTR pszFileName)
{
	if (lstrlen(pszFileName)>=MAX_PATH)
		return false;
	lstrcpy(m_szFileName,pszFileName);
	return true;
}


bool CRecordOption::SetFileExistsOperation(FileExistsOperation Operation)
{
	m_ExistsOperation=Operation;
	return true;
}


bool CRecordOption::SetStopTimeSpec(bool fSpec)
{
	m_fStopTimeSpec=fSpec;
	return true;
}


bool CRecordOption::SetStopTime(unsigned int Time)
{
	m_fStopDateTime=false;
	m_StopTime=Time;
	return true;
}


bool CRecordOption::StartRecord()
{
	SYSTEMTIME stUTC,stLocal;
	FILETIME ft;

	GetSystemTime(&stUTC);
	SystemTimeToTzSpecificLocalTime(NULL,&stUTC,&stLocal);
	SystemTimeToFileTime(&stLocal,&m_ftStartTime);
	m_StartTime=GetTickCount();
	return true;
}


bool CRecordOption::QueryStop() const
{
	if (!m_fStopTimeSpec)
		return false;
	if (m_fStopDateTime) {
		SYSTEMTIME stUTC,stLocal;
		FILETIME ft;

		GetSystemTime(&stUTC);
		SystemTimeToTzSpecificLocalTime(NULL,&stUTC,&stLocal);
		SystemTimeToFileTime(&stLocal,&ft);
		if (CompareFileTime(&ft,&m_ftStopTime)>=0)
			return true;
	} else {
		if (DiffTime(m_StartTime,GetTickCount())>=m_StopTime)
			return true;
	}
	return false;
}


static void SetDateTimeFormat(HWND hDlg,UINT ID)
{
	int Length;
	TCHAR szText[256];

	GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_SSHORTDATE,szText,
														lengthof(szText)-1);
	Length=lstrlen(szText);
	szText[Length++]=' ';
	GetLocaleInfo(LOCALE_USER_DEFAULT,LOCALE_STIMEFORMAT,
										szText+Length,lengthof(szText)-Length);
	DateTime_SetFormat(GetDlgItem(hDlg,ID),szText);
}


BOOL CALLBACK CRecordOption::DlgProc(HWND hDlg,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	static CRecordOption *pThis;

	switch (uMsg) {
	case WM_INITDIALOG:
		{
			int i;

			pThis=reinterpret_cast<CRecordOption*>(lParam);
			SendDlgItemMessage(hDlg,IDC_RECORD_FILENAME,EM_LIMITTEXT,MAX_PATH-1,0);
			SetDlgItemText(hDlg,IDC_RECORD_FILENAME,pThis->m_szFileName);
			static const LPCTSTR pszExistsOperation[] = {
				TEXT("㏑"),TEXT("mF"),TEXT("AԂt")
			};
			for (i=0;i<3;i++)
				SendDlgItemMessage(hDlg,IDC_RECORD_FILEEXISTS,CB_ADDSTRING,0,
							reinterpret_cast<LPARAM>(pszExistsOperation[i]));
			SendDlgItemMessage(hDlg,IDC_RECORD_FILEEXISTS,CB_SETCURSEL,
										(WPARAM)pThis->m_ExistsOperation,0);
			CheckDlgButton(hDlg,IDC_RECORD_STOPSPECTIME,
							pThis->m_fStopTimeSpec?BST_CHECKED:BST_UNCHECKED);
			CheckRadioButton(hDlg,IDC_RECORD_STOPDATETIME,
													IDC_RECORD_STOPREMAINTIME,
				pThis->m_fStopDateTime?
							IDC_RECORD_STOPDATETIME:IDC_RECORD_STOPREMAINTIME);
			if (pThis->m_fStopDateTime) {
				SYSTEMTIME stUTC,st;
				FILETIME ft;
				ULARGE_INTEGER Time1,Time2;

				GetSystemTime(&stUTC);
				SystemTimeToTzSpecificLocalTime(NULL,&stUTC,&st);
				SystemTimeToFileTime(&st,&ft);
				Time1.LowPart=pThis->m_ftStopTime.dwLowDateTime;
				Time1.HighPart=pThis->m_ftStopTime.dwHighDateTime;
				Time2.LowPart=ft.dwLowDateTime;
				Time2.HighPart=ft.dwHighDateTime;
				if (Time1.QuadPart>Time2.QuadPart)
					pThis->m_StopTime=
						(DWORD)((Time1.QuadPart-Time2.QuadPart)/10000);
				else
					pThis->m_StopTime=0;
			} else {
				SYSTEMTIME stUTC,st;
				FILETIME ft;
				ULARGE_INTEGER Time;

				GetSystemTime(&stUTC);
				SystemTimeToTzSpecificLocalTime(NULL,&stUTC,&st);
				st.wSecond=0;
				SystemTimeToFileTime(&st,&ft);
				Time.LowPart=ft.dwLowDateTime;
				Time.HighPart=ft.dwHighDateTime;
				Time.QuadPart+=(ULONGLONG)10000*pThis->m_StopTime;
				pThis->m_ftStopTime.dwLowDateTime=Time.LowPart;
				pThis->m_ftStopTime.dwHighDateTime=Time.HighPart;
			}
			SetDateTimeFormat(hDlg,IDC_RECORD_STOPTIME_TIME);
			SYSTEMTIME st;
			FileTimeToSystemTime(&pThis->m_ftStopTime,&st);
			DateTime_SetSystemtime(GetDlgItem(hDlg,IDC_RECORD_STOPTIME_TIME),
																GDT_VALID,&st);
			SetDlgItemInt(hDlg,IDC_RECORD_STOPTIME_HOUR,
										pThis->m_StopTime/(60*60*1000),FALSE);
			SendDlgItemMessage(hDlg,IDC_RECORD_STOPTIME_HOUR_UD,UDM_SETRANGE,
														0,MAKELPARAM(100,0));
			SetDlgItemInt(hDlg,IDC_RECORD_STOPTIME_MINUTE,
										pThis->m_StopTime/(60*1000)%60,FALSE);
			SendDlgItemMessage(hDlg,IDC_RECORD_STOPTIME_MINUTE_UD,UDM_SETRANGE,
														0,MAKELPARAM(60,0));
			SetDlgItemInt(hDlg,IDC_RECORD_STOPTIME_SECOND,
											pThis->m_StopTime/1000%60,FALSE);
			SendDlgItemMessage(hDlg,IDC_RECORD_STOPTIME_SECOND_UD,UDM_SETRANGE,
														0,MAKELPARAM(60,0));
			if (!pThis->m_fStopTimeSpec) {
				EnableDlgItems(hDlg,IDC_RECORD_STOPDATETIME,
									IDC_RECORD_STOPTIME_SECOND_LABEL,false);
			} else {
				EnableDlgItem(hDlg,IDC_RECORD_STOPTIME_TIME,
													pThis->m_fStopDateTime);
				EnableDlgItems(hDlg,IDC_RECORD_STOPTIME_HOUR,
					IDC_RECORD_STOPTIME_SECOND_LABEL,!pThis->m_fStopDateTime);
			}
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_RECORD_FILENAME_BROWSE:
			{
				OPENFILENAME ofn;
				TCHAR szFileName[MAX_PATH];

				GetDlgItemText(hDlg,IDC_RECORD_FILENAME,szFileName,MAX_PATH);
				ofn.lStructSize=sizeof(OPENFILENAME);
				ofn.hwndOwner=hDlg;
				ofn.lpstrFilter=
					TEXT("TSt@C(*.ts)\0*.ts\0ׂẴt@C\0*.*\0");
				ofn.lpstrCustomFilter=NULL;
				ofn.nFilterIndex=1;
				ofn.lpstrFile=szFileName;
				ofn.nMaxFile=MAX_PATH;
				ofn.lpstrFileTitle=NULL;
				ofn.lpstrInitialDir=NULL;
				ofn.lpstrTitle=TEXT("ۑt@C");
				ofn.Flags=OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
				ofn.lpstrDefExt=TEXT("ts");
#if _WIN32_WINNT>=0x500
				ofn.pvReserved=NULL;
				ofn.dwReserved=0;
				ofn.FlagsEx=0;
#endif
				if (GetSaveFileName(&ofn))
					SetDlgItemText(hDlg,IDC_RECORD_FILENAME,szFileName);
			}
			return TRUE;

		case IDC_RECORD_STOPSPECTIME:
		case IDC_RECORD_STOPDATETIME:
		case IDC_RECORD_STOPREMAINTIME:
			if (IsDlgButtonChecked(hDlg,IDC_RECORD_STOPSPECTIME)==BST_CHECKED) {
				bool fDateTime=IsDlgButtonChecked(hDlg,
										IDC_RECORD_STOPDATETIME)==BST_CHECKED;

				EnableDlgItems(hDlg,IDC_RECORD_STOPDATETIME,
									IDC_RECORD_STOPREMAINTIME,TRUE);
				EnableDlgItem(hDlg,IDC_RECORD_STOPTIME_TIME,fDateTime);
				EnableDlgItems(hDlg,IDC_RECORD_STOPTIME_HOUR,
									IDC_RECORD_STOPTIME_SECOND_LABEL,!fDateTime);
			} else {
				EnableDlgItems(hDlg,IDC_RECORD_STOPDATETIME,
									IDC_RECORD_STOPTIME_SECOND_LABEL,false);
			}
			return TRUE;

		case IDOK:
		case IDCANCEL:
			if (GetWindowTextLength(GetDlgItem(hDlg,IDC_RECORD_FILENAME))==0) {
				MessageBox(hDlg,TEXT("t@C͂ĂB"),NULL,
												MB_OK | MB_ICONEXCLAMATION);
				SetFocus(GetDlgItem(hDlg,IDC_RECORD_FILENAME));
				return TRUE;
			}
			{
				SYSTEMTIME st,stUTC,stLocal;
				FILETIME ft,ftCur;

				DateTime_GetSystemtime(
								GetDlgItem(hDlg,IDC_RECORD_STOPTIME_TIME),&st);
				SystemTimeToFileTime(&st,&ft);
				GetSystemTime(&stUTC);
				SystemTimeToTzSpecificLocalTime(NULL,&stUTC,&stLocal);
				SystemTimeToFileTime(&stLocal,&ftCur);
				if (CompareFileTime(&ft,&ftCur)<=0
						&& IsDlgButtonChecked(hDlg,
									IDC_RECORD_STOPSPECTIME)==BST_CHECKED
						&& IsDlgButtonChecked(hDlg,
									IDC_RECORD_STOPDATETIME)==BST_CHECKED) {
					MessageBox(hDlg,
						TEXT("w肳ꂽ~Ԃɉ߂Ă܂B"),NULL,
												MB_OK | MB_ICONEXCLAMATION);
					SetFocus(GetDlgItem(hDlg,IDC_RECORD_STOPTIME_TIME));
					return TRUE;
				}
				pThis->m_ftStopTime=ft;
			}
			GetDlgItemText(hDlg,IDC_RECORD_FILENAME,pThis->m_szFileName,
																	MAX_PATH);
			pThis->m_ExistsOperation=(FileExistsOperation)
				SendDlgItemMessage(hDlg,IDC_RECORD_FILEEXISTS,CB_GETCURSEL,0,0);
			pThis->m_fStopTimeSpec=IsDlgButtonChecked(hDlg,
										IDC_RECORD_STOPSPECTIME)==BST_CHECKED;
			pThis->m_fStopDateTime=IsDlgButtonChecked(hDlg,
										IDC_RECORD_STOPDATETIME)==BST_CHECKED;
			{
				unsigned int Hour,Minute,Second;

				Hour=GetDlgItemInt(hDlg,IDC_RECORD_STOPTIME_HOUR,NULL,FALSE);
				Minute=GetDlgItemInt(hDlg,IDC_RECORD_STOPTIME_MINUTE,NULL,FALSE);
				Second=GetDlgItemInt(hDlg,IDC_RECORD_STOPTIME_SECOND,NULL,FALSE);
				pThis->m_StopTime=Hour*(60*60*1000)+Minute*(60*1000)+Second*1000;
			}
			EndDialog(hDlg,LOWORD(wParam));
			return TRUE;
		}
		return TRUE;
	}
	return FALSE;
}


bool CRecordOption::ShowDialog(HWND hwndOwner,HINSTANCE hinst)
{
	return DialogBoxParam(hinst,MAKEINTRESOURCE(IDD_RECORDOPTION),hwndOwner,
								DlgProc,reinterpret_cast<LPARAM>(this))==IDOK;
}


bool CRecordOption::DoFileExistsOperation(HWND hwndOwner,LPTSTR pszFileName)
{
	lstrcpy(pszFileName,m_szFileName);
	switch (m_ExistsOperation) {
	case EXISTS_CONFIRM:
		if (PathFileExists(m_szFileName)
				&& MessageBox(hwndOwner,
					TEXT("t@Cɑ݂܂B\n㏑܂?"),
					TEXT("㏑̊mF"),MB_OKCANCEL | MB_ICONQUESTION)!=IDOK)
			return false;
		break;
	case EXISTS_SEQUENCIALNUMBER:
		if (PathFileExists(m_szFileName)) {
			int i;
			TCHAR szFileName[MAX_PATH];
			LPTSTR pszExtension,p;
			int ExtensionOffset;

			pszExtension=PathFindExtension(m_szFileName);
			lstrcpy(szFileName,m_szFileName);
			p=PathFindExtension(szFileName);
			for (i=0;;i++) {
				wsprintf(p,TEXT("%d%s"),i+1,pszExtension);
				if (!PathFileExists(szFileName))
					break;
			}
			lstrcpy(pszFileName,szFileName);
		}
		break;
	}
	return true;
}


BOOL CALLBACK CRecordOption::StopTimeDlgProc(HWND hDlg,UINT uMsg,
												WPARAM wParam,LPARAM lParam)
{
	static CRecordOption *pThis;

	switch (uMsg) {
	case WM_INITDIALOG:
		{
			SYSTEMTIME st;

			pThis=reinterpret_cast<CRecordOption*>(lParam);
			CheckDlgButton(hDlg,IDC_RECORDSTOPTIME_ENABLE,
							pThis->m_fStopTimeSpec?BST_CHECKED:BST_UNCHECKED);
			EnableDlgItem(hDlg,IDC_RECORDSTOPTIME_TIME,pThis->m_fStopTimeSpec);
			SetDateTimeFormat(hDlg,IDC_RECORDSTOPTIME_TIME);
			if (pThis->m_fStopDateTime) {
				FileTimeToSystemTime(&pThis->m_ftStopTime,&st);
			} else {
				ULARGE_INTEGER Time;
				FILETIME ft;

				Time.LowPart=pThis->m_ftStartTime.dwLowDateTime;
				Time.HighPart=pThis->m_ftStartTime.dwHighDateTime;
				Time.QuadPart+=(ULONGLONG)10000*pThis->m_StopTime;
				ft.dwLowDateTime=Time.LowPart;
				ft.dwHighDateTime=Time.HighPart;
				FileTimeToSystemTime(&ft,&st);
			}
			DateTime_SetSystemtime(GetDlgItem(hDlg,IDC_RECORDSTOPTIME_TIME),
																GDT_VALID,&st);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		case IDC_RECORDSTOPTIME_ENABLE:
			EnableWindow(GetDlgItem(hDlg,IDC_RECORDSTOPTIME_TIME),
				IsDlgButtonChecked(hDlg,IDC_RECORDSTOPTIME_ENABLE)==
																BST_CHECKED);
			return TRUE;

		case IDOK:
			{
				bool fStopTimeSpec;
				SYSTEMTIME st;

				fStopTimeSpec=IsDlgButtonChecked(hDlg,
									IDC_RECORDSTOPTIME_ENABLE)==BST_CHECKED;
				if (DateTime_GetSystemtime(
						GetDlgItem(hDlg,IDC_RECORDSTOPTIME_TIME),&st)!=
																GDT_VALID) {
					MessageBox(hDlg,TEXT("Ԃ̎擾G[B"),NULL,
												MB_OK | MB_ICONEXCLAMATION);
					return TRUE;
				}
				FILETIME ft,ftCur;
				SYSTEMTIME stUTC,stLocal;
				ULARGE_INTEGER Time1,Time2;

				SystemTimeToFileTime(&st,&ft);
				GetSystemTime(&stUTC);
				SystemTimeToTzSpecificLocalTime(NULL,&stUTC,&stLocal);
				SystemTimeToFileTime(&stLocal,&ftCur);
				if (pThis->m_fStopTimeSpec) {
					if (CompareFileTime(&ft,&ftCur)<=0) {
						MessageBox(hDlg,
								TEXT("w肳ꂽ~Ԃɉ߂Ă܂B"),
											NULL,MB_OK | MB_ICONEXCLAMATION);
						return TRUE;
					}
				}
				pThis->m_fStopTimeSpec=fStopTimeSpec;
				pThis->m_fStopDateTime=true;
				pThis->m_ftStopTime=ft;
				Time1.LowPart=ftCur.dwLowDateTime;
				Time1.HighPart=ftCur.dwHighDateTime;
				Time2.LowPart=ft.dwLowDateTime;
				Time2.HighPart=ft.dwHighDateTime;
				pThis->m_StopTime=GetTickCount()+
										(DWORD)(Time2.QuadPart-Time1.QuadPart);
			}
		case IDCANCEL:
			EndDialog(hDlg,LOWORD(wParam));
		}
		return TRUE;
	}
	return FALSE;
}


bool CRecordOption::ChangeStopTimeDialog(HWND hwndOwner,HINSTANCE hinst)
{
	return DialogBoxParam(hinst,MAKEINTRESOURCE(IDD_RECORDSTOPTIME),hwndOwner,
						StopTimeDlgProc,reinterpret_cast<LPARAM>(this))==IDOK;
}
